<?php

namespace Opencart\Admin\Controller\Extension\Hepsipay\Payment;

use Hepsipay\HepsiPayGateway;
use Exception;

if ( ! defined( "ALTTANTIRE_MIN_PHP_VER" ) ) {
	define( 'ALTTANTIRE_MIN_PHP_VER', '8.1.0' );
}

class HepsipayVpos extends \Opencart\System\Engine\Controller {
	protected HepsiPayGateway $Gateway;
	private array $error = array();
	private array $success = array();

	public function __construct( $registry ) {
		parent::__construct( $registry );

		include_once DIR_EXTENSION . "hepsipay/system/library/payment-sdk/Gateway.php";
		$this->Gateway = new HepsiPayGateway( $this->config->get( 'payment_hepsipay_vpos_environment' ),
			$this->config->get( 'payment_hepsipay_vpos_merchant_no' ),
			$this->config->get( 'payment_hepsipay_vpos_terminal_no' ),
			$this->config->get( 'payment_hepsipay_vpos_store_key' ) );

	}

	public function index(): void {
		$this->load->language( 'extension/hepsipay/payment/hepsipay_vpos' );
		$this->document->setTitle( $this->language->get( 'heading_title' ) );
		$data['breadcrumbs'] = [];
		$data['breadcrumbs'][] = [
			'text' => $this->language->get( 'text_home' ),
			'href' => $this->url->link( 'common/dashboard', 'user_token=' . $this->session->data['user_token'] )
		];
		$data['breadcrumbs'][] = [
			'text' => $this->language->get( 'text_extension' ),
			'href' => $this->url->link( 'marketplace/extension',
				'user_token=' . $this->session->data['user_token'] . '&type=payment' )
		];
		$data['breadcrumbs'][] = [
			'text' => $this->language->get( 'heading_title' ),
			'href' => $this->url->link( 'extension/hepsipay/payment/hepsipay_vpos',
				'user_token=' . $this->session->data['user_token'] )
		];
		$data['save'] = $this->url->link( 'extension/hepsipay/payment/hepsipay_vpos.save',
			'user_token=' . $this->session->data['user_token'] );
		$data['back'] = $this->url->link( 'marketplace/extension',
			'user_token=' . $this->session->data['user_token'] . '&type=payment' );
		$data['errors_message'] = array(
			'warning'                                  => $this->language->get( 'error_warning' ),
			'payment_hepsipay_vpos_merchant_no'        => $this->language->get( 'error_merchant_no' ),
			'payment_hepsipay_vpos_terminal_no'        => $this->language->get( 'terminal_no' ),
			'payment_hepsipay_vpos_store_key'          => $this->language->get( 'error_store_key' ),
			'payment_hepsipay_vpos_environment'        => $this->language->get( 'error_environment' ),
			'payment_hepsipay_vpos_order_completed_id' => $this->language->get( 'error_order_completed_id' ),
			'payment_hepsipay_vpos_tl_currency'        => $this->language->get( 'error_tl_currency' ),
			'hepsipay_vpos_merchant_general'           => $this->language->get( 'error_hepsipay_vpos_merchant_general' ),
		);

		if ( isset( $this->request->post['payment_hepsipay_vpos_environment'] ) ) {
			$data['payment_hepsipay_vpos_environment'] = trim( $this->request->post['payment_hepsipay_vpos_environment'] );
		} else {
			$data['payment_hepsipay_vpos_environment'] = $this->config->get( 'payment_hepsipay_vpos_environment' );
		}
		if ( isset( $this->request->post['payment_hepsipay_vpos_merchant_no'] ) ) {
			$data['payment_hepsipay_vpos_merchant_no'] = trim( $this->request->post['payment_hepsipay_vpos_merchant_no'] );
		} else {
			$data['payment_hepsipay_vpos_merchant_no'] = $this->config->get( 'payment_hepsipay_vpos_merchant_no' );
		}
		if ( isset( $this->request->post['payment_hepsipay_vpos_terminal_no'] ) ) {
			$data['payment_hepsipay_vpos_terminal_no'] = trim( $this->request->post['payment_hepsipay_vpos_terminal_no'] );
		} else {
			$data['payment_hepsipay_vpos_terminal_no'] = $this->config->get( 'payment_hepsipay_vpos_terminal_no' );
		}
		if ( isset( $this->request->post['payment_hepsipay_vpos_store_key'] ) ) {
			$data['payment_hepsipay_vpos_store_key'] = trim( $this->request->post['payment_hepsipay_vpos_store_key'] );
		} else {
			$data['payment_hepsipay_vpos_store_key'] = $this->config->get( 'payment_hepsipay_vpos_store_key' );
		}
		if ( isset( $this->request->post['payment_hepsipay_vpos_order_completed_id'] ) ) {
			$data['payment_hepsipay_vpos_order_completed_id'] = $this->request->post['payment_hepsipay_vpos_order_completed_id'];
		} else {
			$data['payment_hepsipay_vpos_order_completed_id'] = $this->config->get( 'payment_hepsipay_vpos_order_completed_id' );
		}
		if ( isset( $this->request->post['payment_hepsipay_vpos_tl_currency'] ) ) {
			$data['payment_hepsipay_vpos_tl_currency'] = $this->request->post['payment_hepsipay_vpos_tl_currency'];
		} else {
			$data['payment_hepsipay_vpos_tl_currency'] = $this->config->get( 'payment_hepsipay_vpos_tl_currency' );
		}

		$this->load->model( "localisation/currency" );
		$data['store_currencies'] = $this->model_localisation_currency->getCurrencies();

		$this->load->model( 'localisation/order_status' );
		$data['order_statuses'] = $this->model_localisation_order_status->getOrderStatuses();

		if ( ! $this->config->get( 'payment_hepsipay_vpos_environment' ) ||
		     ! $this->config->get( 'payment_hepsipay_vpos_merchant_no' ) ||
		     ! $this->config->get( 'payment_hepsipay_vpos_terminal_no' ) ||
		     ! $this->config->get( 'payment_hepsipay_vpos_store_key' ) ) {
			$this->error['hepsipay_merchant_general'] = $this->language->get( 'error_empty_install' );
		} else {
			try {
				if ( $this->check_minimum_requirements() ) {
					$this->success[] = $this->language->get( 'success_install' );
				}
			} catch ( Exception $e ) {
				$this->error[] = "<strong>" . $e->getMessage() . "</strong> ". $this->language->get( 'error_fail_install' );
			}
		}

		$this->load->model( 'extension/hepsipay/payment/hepsipay_vpos' );

		$data['errors'] = $this->error;
		$data['success'] = $this->success;
		$data['payment_hepsipay_vpos_status'] = $this->config->get( 'payment_hepsipay_vpos_status' );
		$data['payment_hepsipay_vpos_sort_order'] = $this->config->get( 'payment_hepsipay_vpos_sort_order' );
		$data['header'] = $this->load->controller( 'common/header' );
		$data['column_left'] = $this->load->controller( 'common/column_left' );
		$data['footer'] = $this->load->controller( 'common/footer' );
		$data['system_info'] = [
			"hepsipay_plugin"   => $this->model_extension_hepsipay_payment_hepsipay_vpos->get_extension_version(),
			"opencart"          => defined( 'VERSION' ) ? VERSION : "N/A",
			"php_compatibility" => version_compare( phpversion(), ALTTANTIRE_MIN_PHP_VER, '>=' ),
			"php_version"       => phpversion(),
			"curl"              => function_exists( 'curl_init' ),
			"file_get_contents" => function_exists( 'file_get_contents' ) && ini_get( 'allow_url_fopen' ),
		];


		$this->response->setOutput( $this->load->view( 'extension/hepsipay/payment/hepsipay_vpos', $data ) );
	}

	public function save(): void {
		$this->load->language( 'extension/hepsipay/payment/hepsipay_vpos' );
		$json = [];
		if ( ! $this->user->hasPermission( 'modify', 'extension/hepsipay/payment/hepsipay_vpos' ) ) {
			$json['error'] = $this->language->get( 'error_permission' );
		}
		$this->request->post['payment_hepsipay_vpos_merchant_no'] = intval( $this->request->post['payment_hepsipay_vpos_merchant_no'] );
		$validate = $this->validate();
		if ( $validate !== TRUE ) {
			$json['error'] = $validate;
		}
		if ( ! $json && $validate === TRUE ) {
			$this->load->model( 'setting/setting' );
			$this->load->model( 'extension/hepsipay/payment/hepsipay_vpos' );
			$credential_result = $this->model_extension_hepsipay_payment_hepsipay_vpos->checkCredential( $this->request->post );
			if ( $credential_result !== TRUE ) {
				$json['error'] = "Hepsipay kurulum bilgileri hatalı!";
				/*$this->model_setting_setting->editSetting( 'payment_hepsipay',
					[ "payment_hepsipay_vpos_status" => 0 ] );*/
			} else {
				$this->model_setting_setting->editSetting( 'payment_hepsipay', $this->request->post );
				$json['success'] = $this->language->get( 'text_success' );
			}
		}
		$this->response->addHeader( 'Content-Type: application/json' );
		$this->response->setOutput( json_encode( $json ) );
	}

	protected function validate(): bool|string {
		if ( ! $this->request->post['payment_hepsipay_vpos_environment'] ) {
			return $this->language->get( 'error_environment' );
		}
		if ( ! $this->request->post['payment_hepsipay_vpos_merchant_no'] ) {
			return $this->language->get( 'error_merchant_no' );
		}
		if ( ! $this->request->post['payment_hepsipay_vpos_terminal_no'] ) {
			return $this->language->get( 'terminal_no' );
		}
		if ( ! $this->request->post['payment_hepsipay_vpos_store_key'] ) {
			return $this->language->get( 'error_store_key' );
		}
		if ( ! $this->request->post['payment_hepsipay_vpos_order_completed_id'] ) {
			return $this->language->get( 'error_order_completed_id' );
		}
		if ( ! $this->request->post['payment_hepsipay_vpos_tl_currency'] ) {
			return $this->language->get( 'error_tl_currency' );
		}

		return TRUE;
	}

	public function install(): void {
		if ( $this->user->hasPermission( 'modify', 'extension/payment' ) ) {
			$this->load->model( 'extension/hepsipay/payment/hepsipay_vpos' );
			$this->model_extension_hepsipay_payment_hepsipay_vpos->install();
		}
	}

	public function uninstall(): void {
		if ( $this->user->hasPermission( 'modify', 'extension/payment' ) ) {
			$this->load->model( 'extension/hepsipay/payment/hepsipay_vpos' );
			$this->model_extension_hepsipay_payment_hepsipay_vpos->uninstall();
		}
	}

	function check_minimum_requirements(): bool {
		if ( version_compare( phpversion(), ALTTANTIRE_MIN_PHP_VER, '<' ) ) {
			$this->error[] = "Web sitenizde PHP " . phpversion() . " Versiyonunu kullanıyorsunuz. Hepsipay Sanal POS eklentisi, PHP " . ALTTANTIRE_MIN_PHP_VER . " ve üzeri versiyonda çalışır.";

			return FALSE;
		}
		if ( ! function_exists( 'curl_init' ) && ! function_exists( 'file_get_contents' ) && ! ini_get( 'allow_url_fopen' ) ) {
			$this->error[] = "Sunucunuzda cURL kurulu ya da aktif değil. Hepsipay Sanal POS'u kullanabilmeniz için sunucunuzda cURL kurulu olmalıdır.";

			return FALSE;
		}

		return TRUE;
	}
}

