<?php

namespace Opencart\Catalog\Controller\Extension\Hepsipay\Payment;

use Hepsipay\HepsiPayGateway;
use Hepsipay\CookieSetter;

class HepsipayVpos extends \Opencart\System\Engine\Controller
{
    protected object $Gateway;
    protected int $installment;
    protected string $threed_session_id;
    protected string $payment_url;
    protected bool $has_installment_auth;

    protected array $card = [];

    public function __construct($registry)
    {
        parent::__construct($registry);

        $this->load->model("extension/hepsipay/payment/hepsipay_vpos");

        include_once DIR_EXTENSION."hepsipay/system/library/payment-sdk/Gateway.php";
        include_once DIR_EXTENSION."hepsipay/system/library/payment-sdk/CookieSetter.php";

        $this->Gateway = new HepsiPayGateway(
            $this->config->get("payment_hepsipay_vpos_environment"),
            $this->config->get("payment_hepsipay_vpos_merchant_no"),
            $this->config->get("payment_hepsipay_vpos_terminal_no"),
            $this->config->get("payment_hepsipay_vpos_store_key")
        );
    }

    public function index(): string
    {
        $this->load->language("extension/hepsipay/payment/hepsipay_vpos");

        if (isset($this->session->data["payment_method"])) {

            $this->load->model("localisation/currency");
            if ($this->session->data["currency"] !== $this->config->get("payment_hepsipay_vpos_tl_currency")) {
                return $this->load->view(
                    "extension/hepsipay/payment/hepsipay_vpos",
                    ["verified" => false, "text_description" => $this->language->get('error_foreign_currency')]
                );
            }

            $this->document->addScript('https://images.hepsiburada.net/hepsipay/packages/pf/hepsipay-0.2.7.min.js');

            $this->set_same_site_policy();

            $data["verified"]   = false;
            $data["frame_url"]  = null;
            $data["order_data"] = $this->get_order_data();
            $data["language"]   = $this->config->get("config_language");

            if ( ! $data["order_data"]["success"]) {
                $data["text_description"] = $data["order_data"]["message"];

                return $this->load->view(
                    "extension/hepsipay/payment/hepsipay_vpos",
                    $data
                );
            }

            $this->Gateway->set_amount($data["order_data"]["payment_amount"]);

            $this->Gateway->set_account_key(
                trim($data["order_data"]["info"]["email"])
            );

            $this->Gateway->set_merchant_callback_url(
                $this->url->link(
                    "extension/hepsipay/payment/hepsipay_vpos.callback_handler&language=".
                    $data["language"],
                    "",
                    "SSL"
                )
            );
            $this->Gateway->set_merchant_order_number(
                $data["order_data"]["info"]["order_id"]
            );
            $this->Gateway->set_is_guest_user(true);
            $this->Gateway->set_force_3ds(true);
            $this->Gateway->set_allowed_installments([
                1,
                2,
                3,
                4,
                5,
                6,
                7,
                8,
                9,
                10,
                11,
                12,
            ]);

            $this->Gateway->set_frame_type(1);

            $this->Gateway->set_basket_item([
                "Product"                => "Ürün Toplamı",
                "Price"                  => $data["order_data"]["payment_amount"],
                "SubMerchantMemberId"    => null,
                "SubMerchantMemberPrice" => null,
                "CategoryId"             => null,
                "ExternalId"             => $data["order_data"]["info"]["order_id"],
            ]);

            try {
                // Check Availability
                $frame_check_availability_response = $this->Gateway->check_availability();
                if ($frame_check_availability_response->Success === true) {
                    $frame_init_response = $this->Gateway->frame_init();

                    if ($frame_init_response && $frame_init_response->Success === true) {
                        $frame_url         = $frame_init_response->FrameUrl;
                        $data["verified"]  = true;
                        $data["frame_url"] = $frame_url;
                    } else {
                        $data["text_description"] =
                            $frame_init_response->Message;
                    }

                } else {
                    $data["text_description"] =
                        $frame_check_availability_response->Message;
                }
            } catch (\InvalidArgumentException $e) {
                $data["text_description"] = (string) $e->getMessage();
            }

            return $this->load->view(
                "extension/hepsipay/payment/hepsipay_vpos",
                $data
            );
        }

        return "";
    }

    public function get_order_data()
    {
        if ( ! isset($_SERVER["HTTP_REFERER"])) {
            die("NOT FOUND");
        }

        $data            = ["success" => false];
        $data["message"] = null;

        $this->load->model("checkout/order");
        $this->load->model("localisation/currency");
        $this->load->language("extension/payment/hepsipay");

        $currency = $this->model_localisation_currency->getCurrencyByCode(
            $this->session->data["currency"]
        );

        $order_info = $this->model_checkout_order->getOrder(
            $this->session->data["order_id"]
        );

        if ($order_info) {
            if ($order_info["order_status_id"] == 0) {
                if ($order_info["total"] == 0) {
                    $data["message"] = $this->language->get(
                        "error_invalid_amount"
                    );
                }
            } else {
                $data["message"] =
                    "OrderID: ".
                    $this->session->data["order_id"].
                    " ".
                    $this->language->get("error_order_already_paid");
            }
        } else {
            $data["message"] = $this->language->get("error_order_not_found");
        }

        if ($data["message"]) {
            return $data;
        }

        $data["success"] = true;

        $data["info"]           = $order_info;
        $data["products"]       = $this->model_checkout_order->getProducts(
            $this->session->data["order_id"]
        );
        $data["payment_amount"] = intval(
            $this->model_extension_hepsipay_payment_hepsipay_vpos->roundUp(
                $order_info["total"] * $currency["value"]
            ) * 100
        );

        return $data;
    }

    private function set_same_site_policy()
    {
        $path   = ini_get("session.cookie_path");
        $domain = $this->request->server["HTTP_HOST"];
        foreach ($_COOKIE as $k => $v) {
            CookieSetter::setcookie($k, $v, [
                "secure"   => true,
                "samesite" => "None",
                "path"     => $path,
                "domain"   => $domain,
            ]);
        }
    }


    public function callback_handler()
    {
        $css    = HTTP_SERVER."extension/hepsipay/catalog/view/template/payment/spinner.css";
        $output = '<!doctypehtml><html lang=en><meta charset=UTF-8><title>Spinner</title>'
                  .'<link href="'.$css.'" id=alttantire-gateway-payment-spinner-css '
                  .'media=all rel=stylesheet><style>body,html{height:100%;margin:0;padding:0;'
                  .'background-color:#fff;display:flex;justify-content:center;align-items:center;'
                  .'font-family:Arial,sans-serif}.loader{width:100px;height:100px;display:flex;'
                  .'justify-content:center;align-items:center;color:#fff;font-size:20px}</style>'
                  .'<div class=loader><svg class=spinner viewBox="0 0 100 100">'
                  .'<circle class=spinner__path cx=50 cy=50 r=40 stroke="rgba(244, 63, 94, 1)"/>'
                  .'<circle class=spinner__path cx=50 cy=50 r=35 stroke="rgba(245, 158, 11, 1)"/>'
                  .'<circle class=spinner__path cx=50 cy=50 r=30 stroke="rgba(16, 185, 129, 1)"/>'
                  .'<circle class=spinner__path cx=50 cy=50 r=25 stroke="rgba(119, 35, 219, 1)"/>'
                  .'</svg></div>';

        echo $output;
        exit;
    }

    public function callback()
    {
        $hepsipay_payment_data = json_decode(
            filter_input(INPUT_POST, "hepsipay_payment_data", FILTER_DEFAULT)
        );

        if ( ! is_object($hepsipay_payment_data) || ! isset($hepsipay_payment_data->token)) {
            return null;
        }

        $token    = $hepsipay_payment_data->token;
        $order_id = $hepsipay_payment_data->MerchantOrderNumber;

        $payment_response = $this->Gateway->payment_status($token);

        if ( ! is_object($payment_response)) {
            $this->session->data["error"] = "Hatalı işlem..";
            echo json_encode([
                "redirect_url" => $this->url->link(
                    "checkout/failure",
                    "language=".$this->config->get("config_language"),
                    true
                ),
                "message"      => $this->session->data["error"],
            ]);
            exit();
        }

        if (in_array($payment_response->Status, [4, 5, 12, 14])) {
            $this->load->model("checkout/order");
            $this->load->model("localisation/currency");

            $currency = $this->model_localisation_currency->getCurrencyByCode(
                $this->session->data["currency"]
            );

            $order_info = $this->model_checkout_order->getOrder($order_id);

            if ($order_info["order_status"] !== $this->config->get('payment_hepsipay_vpos_order_completed_id')) {
                $interest_rate = $payment_response->InterestAmount;

                $store_currency         = $this->config->get('config_currency');
                $calculated_order_total = $this->currency->convert($payment_response->Amount / 100, $currency["code"],
                    $store_currency);

                $this->model_extension_hepsipay_payment_hepsipay_vpos->updateOrderTotal(
                    $order_id,
                    $calculated_order_total,
                    $interest_rate
                );

                $notes[] = "Ödeme Türü: ".$payment_response->PaymentTypeDescription ?? "";
                $notes[] = "Taksit: ".
                           ($payment_response->InstallmentCount > 1 ? $payment_response->InstallmentCount : "Tek Çekim");
                $notes[] = "Vade Farkı: ".$interest_rate." TL";
                $notes[] =
                    "Ödenen Toplam Tutar: ".
                    $payment_response->Amount / 100 .
                    " TL";
                $notes[] = "Not: ".$payment_response->StatusDescription;

                $this->model_checkout_order->addHistory(
                    $order_id,
                    $this->config->get("payment_hepsipay_vpos_order_completed_id"),
                    implode("<br/>", $notes),
                    false
                );
            }


            echo json_encode([
                "redirect_url" => $this->url->link(
                    "checkout/success",
                    "language=".$this->config->get("config_language"),
                    true
                ),
                "message"      => "",
            ]);
            exit();
        }

        $this->session->data["error"] = $payment_response->StatusDescription;

        echo json_encode([
            "redirect_url" => $this->url->link(
                "checkout/failure",
                "language=".$this->config->get("config_language"),
                true
            ),
            "message"      => $this->session->data["error"],
        ]);
        exit();
    }
}
